<?php
/**
 * Api for SOAP connection made easier, WSDL only
 *
 * All your soap function should be accessible in this object directly
 * You can xclass arc_soapapi to implement your own soap call if you need particular calling method over some functions
 *
 * @author	Christophe Monard (Archriss) <cmonard@archriss.com>
 *
 * calling method:
 *  require_once(t3lib_extMgm::extPath('arc_soapapi').'class.tx_arcsoapapi.php');
 *	$soapLib = t3lib_div::makeInstance('tx_arcsoapapi');
 *  $return = $soapLib->myWebServiceFunction($parameter1, $parameter2);
 *  $return = $soapLib->myWebServiceFunction($parameterArray);
 *  // list function - if SHOWFUNC is checked
 *  echo $soapLib->listFunc;
 *  // display last Request / Respond - if TRACE is checked
 *  echo $soapLib->lastRR;
 *  // display last error
 *  echo $soapLib->error;
 *
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *   46: class tx_arcsoapapi
 *   92:    protected function init($conf)
 *  114:    protected function getValue($conf, $val)
 *  142:    public function __construct()
 *  159:    public function __destruct()
 *  163:    public function __call($method, $arguments)
 *  188:    private function SOAP_RR_debug()
 *  196:	protected function sendMail($param = array())
 *  244:	private function getHook($hookName, &$hookConf = array())
 *  252:	private function userFunc($funcName, &$funcConf)
 *
 * TOTAL FUNCTIONS: 9
 *
 */

/**
 * General SOAP client class
 *
 * @author	Christophe Monard <cmonard@archriss.com>
 * @package	TYPO3
 * @subpackage	tx_arcsoapapi
 */
class tx_arcsoapapi {
	protected $error_mail = '';
	protected $bindcopie = '';
	protected $wsdl = FALSE;
	protected $client;
	protected $finalURI = '';
	protected $uri = array(
		0 => 'http', 
		1 => '',
		2 => '://', 
		3 => '',
		4 => ':', 
		5 => '',
		6 => '/',
		7 => ''
	);
	protected $allowedUriOption = array(
		'SSL' => array('field' => 1, 'type' => 'ssl'),
		'HOST' => array('field' => 3, 'type' => ''),
		'PORT' => array('field' => 5, 'type' => 'integer'),
		'SCHEME' => array('field' => 7, 'type' => ''),
	);
	protected $soapConf = array();
	protected $allowedSoapOption = array(
		'SOAP' => array('field' => 'soap_version', 'type' => ''),
		'LOGIN' => array('field' => 'login', 'type' => 'none'),
		'PASSWORD' => array('field' => 'password', 'type' => 'none'),
		'PROXYHOST' => array('field' => 'proxy_host', 'type' => 'none'),
		'PROXYPORT' => array('field' => 'proxy_port', 'type' => 'none'),
		'PROXYUSERNAME' => array('field' => 'proxy_login', 'type' => 'none'),
		'PROXYPASSWORD' => array('field' => 'proxy_password', 'type' => 'none'),
		'COMPRESSION' => array('field' => 'compression', 'type' => 'zip'),
		'EXCEPTIONS' => array('field' => 'exceptions', 'type' => ''),
		'TRACE' => array('field' => 'trace', 'type' => 'boolean'),
		'CACHE' => array('field' => 'cache_wsdl', 'type' => ''),
	);
	protected $allowedProperties = array(
		'WSDL' => array('field' => 'wsdl', 'type' => 'boolean'),
		'SHOWFUNC' => array('field' => 'showfunc', 'type' => 'boolean'),
		'ERROR_MAIL' => array('field' => 'error_mail', 'type' => 'mail'),
		'BIND_MAIL' => array('field' => 'bindcopie', 'type' => 'mail'),
	);
	public $error = '';
	public $listFunc = '';
	public $lastRR = '';

	protected function init($conf) {
		$this->getHook('preInit');
		if (is_null($conf)) {
			// global $TYPO3_CONF_VARS;
			$conf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['arc_soapapi']);
		}
		if (is_array($conf)) {
			$soapOptionKey = array_keys($this->allowedSoapOption);
			$uriOptionKey = array_keys($this->allowedUriOption);
			$propertiesKey = array_keys($this->allowedProperties);
			foreach ($conf as $confKey => $confValue) {
				$confKey = strtoupper($confKey);
				if (in_array($confKey, $soapOptionKey)) $this->soapConf[$this->allowedSoapOption[$confKey]['field']] = $this->getValue($this->allowedSoapOption[$confKey]['type'], $confValue);
				elseif (in_array($confKey, $uriOptionKey)) $this->uri[$this->allowedUriOption[$confKey]['field']] = $this->getValue($this->allowedUriOption[$confKey]['type'], $confValue);
				elseif (in_array($confKey, $propertiesKey)) $this->$this->allowedProperties[$confKey]['field'] = $this->getValue($this->allowedProperties[$confKey]['type'], $confValue);
			}
			if ($this->uri[5] == '') unset($this->uri[4]);
			$this->finalURI = implode('', $this->uri);
		}
		$this->getHook('postInit');
	}

	protected function getValue($conf, $val) {
		switch ($conf) {
			case 'none':
				$return = ($val!=''?$val:NULL);
				break;
			case 'integer':
				$return = intval($val);
				break;
			case 'boolean':
				$return = $val?TRUE:FALSE;
				break;
			case 'mail':
				$return = t3lib_div::validEmail($val)?$val:'';
				break;
			case 'ssl':
				$return = ($val?'s':'');
				break;
			case 'zip':
				$return = SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP | 5;
				break;
			default:
				$return = $val;
				break;
		}
		$this->getHook('postValue', array('returnValue' => $return, 'confType' => $conf));
		return $return;
	}

	public function __construct($confOverride = NULL) {
		$this->init($confOverride);
		try {
			$this->client = new SoapClient(($this->wsdl?$this->finalURI.'?WSDL':NULL), array_merge(array('location' => $this->finalURI), $this->soapConf));
		} catch (Exception $e) {
			$this->error = htmlspecialchars($e->getMessage(), ENT_QUOTES);
			if ($this->error != '')
				$this->sendMail(array('subject' => 'Constructor error', 'body' => '<h2>Debug</h2><pre>'.$this->error.'</pre>'));
		}
		if (is_object($this->client) && SHOWFUNC) {
			$this->listFunc = '<pre>';
			$this->listFunc.= "Function list :\n";
			$this->listFunc.= print_r($this->client->__getFunctions(), 1);
			$this->listFunc.= '</pre>';
		}
	}

	public function __destruct() {
		unset($this->client);
	}

	public function __call($method, $arguments) {
		$func = create_function('$a,$b,$c', '
		if ($c) {
			try {
				$return = $a->client->'.$method.'($b);
			} catch (Exception $e) {
				$a->error = htmlspecialchars($e->getMessage(), ENT_QUOTES);
				$return = FALSE;
			}
		} else {
			$return = $a->client->'.$method.'($b);
			if (!$return->'.$method.'Result) {
				$a->error = htmlspecialchars($return->getMessage(), ENT_QUOTES);
				$return = FALSE;
			}
		}
		if ($a->error != \'\')
			$a->sendMail(array(\'subject\' => \'Method error: '.$method.'\', \'body\' => \'<h2>Debug</h2><pre>\'.$a->error.\'</pre>\'));
		return $return;
		');
		$return = $func($this, (is_array($arguments[0])?$arguments[0]:implode(',', $arguments)), EXCEPTIONS);
		if (TRACE) $this->lastRR = $this->SOAP_RR_debug();
		return $return;
	}

	private function SOAP_RR_debug() {
		$return = '<pre>';
		$return.= "Request :\n".htmlspecialchars($this->client->__getLastRequest()) ."\n";
		$return.= "Response:\n".htmlspecialchars($this->client->__getLastResponse())."\n\n";
		$return.= '</pre>';
		return $return;
	}

	protected function sendMail($param = array()) {
		if ($this->error_mail != '') { // send mail only if we have recipient
			$recipient = t3lib_div::trimExplode(',', $this->error_mail);
			$bindcopie = t3lib_div::trimExplode(',', $this->bindcopie);
			$returnPath = $recipient[0];
			if (t3lib_div::int_from_ver(TYPO3_version) >= 4005000) { // TYPO3 4.5 and more
				if (count($param > 0) && isset($param['body'])) {
					$mail = t3lib_div::makeInstance('t3lib_mail_Message');
					foreach ($recipient as $to_email)
						$mail->addTo($to_email, NULL);
					if ($this->bindcopie != '')
						foreach ($bindcopie as $bcc_email)
							$mail->addBcc($bcc_email, NULL);
					if ($returnPath != '')
						$mail->setReturnPath($returnPath);
					if (isset($param['subject']))
						$mail->setSubject($param['subject']);
					$mail->setBody($param['body'], 'text/html');
					$sent = $mail->send();
					$fail = $mail->getFailedRecipients();
					return array('sent' => $sent, 'fail' => $fail);
				} else return FALSE;
			} else { // old method still working for compatibility
				if (count($param > 0) && isset($param['body'])) {
					$mailer=t3lib_div::makeInstance('t3lib_htmlmail');
					$mailer->start();
					$mailer->use8Bit();
					$mailer->subject = $param['subject']?$param['subject']:'';
					foreach ($recipient as $to_email)
						$mailer->recipient.= $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset']=='utf-8'?utf8_decode($to_email):$to_email;
					if ($this->bindcopie != '')
						foreach ($bindcopie as $bcc_email)
							$mailer->recipient_blindcopy.= $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset']=='utf-8'?utf8_decode($bcc_email):$bcc_email;
					if ($returnPath != '') {
						$mailer->forceReturnPath = 1;
						$mailer->returnPath = $returnPath;
					}
					$mailer->setPlain($mailer->encodeMsg($param['body']));

					$mailer->setHeaders();
					$mailer->setContent();

					return $mailer->sendTheMail();
				} else return FALSE;
			}
		}
	}

	private function getHook($hookName, $hookConf = array()) {
		if (is_array($TYPO3_CONF_VARS['SC_OPTIONS']['arc_soapapi/class.tx_arcsoapapi.php'][$hookName])) {
			foreach($TYPO3_CONF_VARS['SC_OPTIONS']['arc_soapapi/class.tx_arcsoapapi.php'][$hookName] as $funcRef)
				$this->userFunc($funcRef, &$hookConf);
			return true;
		} else return FALSE;
	}

	private function userFunc($funcName, &$funcConf) {
		$funcConf['parentObj'] = &$this->caller;
		return t3lib_div::callUserFunction($funcName, $funcConf, $this);
	}
}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/arc_soapapi/class.tx_arcsoapapi.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/arc_soapapi/class.tx_arcsoapapi.php']);
}
?>